"""
Parametric Study
================

`skewExample.py` from `Using a script to perform a parametric study <https://help.3ds.com/2021/English/DSSIMULIA_Established/SIMACAECMDRefMap/simacmd-c-intexaaclskewparametric.htm?contextscope=all&id=c78341a7f00b493caa61a4c530165d52>`_.

This script performs a parameter study of element type versus 
skew angle. For more details, see Problem 2.3.4 in the 
Abaqus Benchmarks manual.

Before executing this script you must fetch the appropriate 
files: abaqus fetch job=skewExample
       abaqus fetch job=skewExampleUtils.py

Run the following command before running this script:

.. code-block:: sh

    abaqus fetch job=viewer_tutorial

The following shows the contents of the script skewExample.py. The parametric study does the following:

- Opens the model database and creates variables that refer to the part, the assembly, and the part instance stored in `Model-1`.
- Creates variables that refer to the four faces and the nine vertices in the instance of the planar shell part.
- Skews the plate by modifying the angular dimension in the sketch of the base feature.
- Defines the logical corners of the four faces, and generates a structured mesh.
- Runs the analysis for a range of angles using two element types for each angle.
- Calculates the maximum moment and displacement at the center of the shell.
- Displays **X - Y** plots in separate viewports of the following:

  - Displacement versus skew angle
  - Maximum bending moment versus skew angle
  - Minimum bending moment versus skew angle

  The theoretical results are also plotted.
"""
from abaqus import *
from abaqusConstants import *
import visualization

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getResults():

    """
    Retrieve the displacement and calculate the minimum 
    and maximum bending moment at the center of plate.
    """

    from visualization import ELEMENT_NODAL

    # Open the output database.
    
    odb = visualization.openOdb('skew.odb')
    centerNSet = odb.rootAssembly.nodeSets['CENTER']
    frame = odb.steps['Step-1'].frames[-1]
    
    # Retrieve Z-displacement at the center of the plate.
    
    dispField = frame.fieldOutputs['U']
    dispSubField = dispField.getSubset(region=centerNSet)
    disp = dispSubField.values[0].data[2]

    # Average the contribution from each element to the moment,
    # then calculate the minimum and maximum bending moment at
    # the center of the plate using Mohr's circle.
    
    momentField = frame.fieldOutputs['SM']
    momentSubField = momentField.getSubset(region=centerNSet, 
        position=ELEMENT_NODAL)
    m1, m2, m3 = 0, 0, 0
    for value in momentSubField.values:
        m1 = m1 + value.data[0]
        m2 = m2 + value.data[1]
        m3 = m3 + value.data[2]
    numElements = len(momentSubField.values)    
    m1 = m1 / numElements
    m2 = m2 / numElements
    m3 = m3 / numElements
    momentA = 0.5 * (abs(m1) + abs(m2))
    momentB = sqrt(0.25 * (m1 - m2)**2 + m3**2)
    maxMoment = momentA + momentB
    minMoment = momentA - momentB

    odb.close()
    
    return disp, maxMoment, minMoment

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createXYPlot(vpOrigin, vpName, plotName, data):
    
    """
    Display curves of theoretical and computed results in
    a new viewport.
    """

    from visualization import  USER_DEFINED
    
    vp = session.Viewport(name=vpName, origin=vpOrigin, 
        width=150, height=100)
    xyPlot = session.XYPlot(plotName)
    chart = xyPlot.charts.values()[0]
    curveList = []
    for elemName, xyValues in sorted(data.items()):
        xyData = session.XYData(elemName, xyValues)
        curve = session.Curve(xyData)
        curveList.append(curve)
    chart.setValues(curvesToPlot=curveList)
    chart.axes1[0].axisData.setValues(useSystemTitle=False,title='Skew Angle')
    chart.axes2[0].axisData.setValues(useSystemTitle=False,title=plotName)
    vp.setValues(displayedObject=xyPlot)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createModel():

    """
    Create the skew example model, including material, step, load, bc, and job.
    """

    import regionToolset, part, step, mesh

    # Create the Plate
    m = mdb.models['Model-1']
    s = m.ConstrainedSketch(name='__profile__', sheetSize=5.0)
    g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
    s.sketchOptions.setValues(sheetSize=5.0, gridSpacing=0.1, grid=ON, 
                            gridFrequency=2, constructionGeometry=ON,
                            dimensionTextHeight=0.1, decimalPlaces=2)
    s.setPrimaryObject(option=STANDALONE)
    s.rectangle(point1=(0.0, 0.0), point2=(1.0, 1.0))
    s.delete(objectList=(c[21], c[18], c[19], c[20]))
    s.HorizontalConstraint(entity=g.findAt((0.5, 0.0)))
    s.FixedConstraint(entity=v.findAt((0.0, 0.0)))
    s.FixedConstraint(entity=v.findAt((1.0, 0.0)))
    s.ParallelConstraint(entity1=g.findAt((0.0, 0.5)),
                        entity2=g.findAt((1.0,0.5)))
    s.AngularDimension(line1=g.findAt((0.0, 0.5)), line2=g.findAt((0.5, 0.0)), 
                    textPoint=(0.2, 0.2), value=90.0)
    p = m.Part(name='Plate', dimensionality=THREE_D, type=DEFORMABLE_BODY)
    p.BaseShell(sketch=s)
    s.unsetPrimaryObject()
    vp = session.viewports['Viewport: 1']
    vp.setValues(displayedObject=p)
    del mdb.models['Model-1'].sketches['__profile__']

    # Create the Steel material
    m.Material('Steel')
    m.materials['Steel'].Elastic(table=((30.e6, 0.3), ))
    m.HomogeneousShellSection(name='Shell', preIntegrate=OFF, material='Steel',
                            thickness=0.01, poissonDefinition=DEFAULT, 
                            temperature=GRADIENT, integrationRule=SIMPSON, numIntPts=5)

    # Assign Steel to the plate
    p = mdb.models['Model-1'].parts['Plate']
    region =(None, None, p.faces, None)
    p.SectionAssignment(region=region, sectionName='Shell')


    # Create the assembly
    a = m.rootAssembly
    vp.setValues(displayedObject=a)
    a.DatumCsysByDefault(CARTESIAN)
    a.Instance(name='Plate-1', part=p, dependent=OFF)
    pi = a.instances['Plate-1']

    # Create the step
    m.StaticStep(name='Step-1', previous='Initial',
                description='Apply pressure', timePeriod=1, initialInc=1)
    vp.assemblyDisplay.setValues(step='Step-1')
    m.fieldOutputRequests['F-Output-1'].setValues(frequency=1, variables=('U',))
    m.FieldOutputRequest(name='F-Output-2', createStepName='Step-1',
                        variables=('SF',), position=NODES)
    del mdb.models['Model-1'].historyOutputRequests['H-Output-1']

    # Create the displacement BC
    e = pi.edges
    edges = e.findAt(((0.25, 0.0, 0.0), ), ((1.0, 0.25, 0.0), ),
                    ((0.75, 1.0, 0.0), ), ((0.0, 0.75, 0.0), ), )
    region =(None, edges, None, None)
    m.DisplacementBC(name='Pinned', createStepName='Step-1', region=region,
                    u1=0.0, u2=0.0, u3=0.0)

    # Create the Pressure load
    s1 = pi.faces
    side1Faces1 = s1.findAt(((0.333333333333333, 0.333333333333333, 0.0),
                            (0.0, 0.0, 1.0), ),)
    region = regionToolset.Region(side1Faces=side1Faces1)
    m.Pressure(name='Load-1', createStepName='Step-1', region=region,
            distributionType=UNIFORM, magnitude=1.0, amplitude=UNSET)

    # Partition the face
    f1, e1 = pi.faces, pi.edges
    faces = (f1.findAt(coordinates=(0.33333333333, 0.33333333333, 0.0)), )
    pt1 = pi.InterestingPoint(edge=e1.findAt(coordinates=(
        0.0, 0.75, 0.0)), rule=MIDDLE)
    pt2 = pi.InterestingPoint(edge=e1.findAt(coordinates=(
        1.0, 0.25, 0.0)), rule=MIDDLE)
    a.PartitionFaceByShortestPath(faces=faces, point1=pt1, point2=pt2)
    faces = (f1.findAt(coordinates=(0.33333333333, 0.66666666667, 0.0)), 
            f1.findAt(coordinates=(0.66666666667, 0.33333333333, 0.0)))
    pt1 = pi.InterestingPoint(edge=e1.findAt(coordinates=(
        0.75, 1.0, 0.0)), rule=MIDDLE)
    pt2 = pi.InterestingPoint(edge=e1.findAt(coordinates=(
        0.25, 0.0, 0.0)), rule=MIDDLE)
    a.PartitionFaceByShortestPath(faces=faces, point1=pt1, point2=pt2)

    # Create the Geometry set CENTER
    verts = pi.vertices.findAt(((0.5, 0.5, 0.0), ))
    a.Set(name='CENTER', vertices=verts)

    # Create the mesh
    a.seedPartInstance(regions=(pi,), size=0.25)
    a.generateMesh(regions=(pi,))

    # Create the job
    mdb.Job(name='skew', model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE,
            description='', userSubroutine='', numCpus=1, scratch='',
            echoPrint=OFF, modelPrint=OFF, contactPrint=OFF, historyPrint=OFF)

import part
import mesh
from mesh import S4, S8R, STANDARD, STRUCTURED
import job

# Create a list of angle parameters and a list of
# element type parameters.

angles = [90, 80, 60, 40, 30]
elemTypeCodes = [S4, S8R]

# Open the model database.
openMdb('skew.cae')

model = mdb.models['Model-1']
part = model.parts['Plate']
feature = part.features['Shell planar-1']
assembly = model.rootAssembly
instance = assembly.instances['Plate-1']
job = mdb.jobs['skew']

allFaces = instance.faces
regions =(allFaces[0], allFaces[1], allFaces[2], allFaces[3])
assembly.setMeshControls(regions=regions,
    technique=STRUCTURED)
face1 = allFaces.findAt((0.,0.,0.), )
face2 = allFaces.findAt((0.,1.,0.), )
face3 = allFaces.findAt((1.,1.,0.), )
face4 = allFaces.findAt((1.,0.,0.), )
allVertices = instance.vertices
v1 = allVertices.findAt((0.,0.,0.), )
v2 = allVertices.findAt((0.,.5,0.), )
v3 = allVertices.findAt((0.,1.,0.), )
v4 = allVertices.findAt((.5,1.,0.), )
v5 = allVertices.findAt((1.,1.,0.), )
v6 = allVertices.findAt((1.,.5,0.), )
v7 = allVertices.findAt((1.,0.,0.), )
v8 = allVertices.findAt((.5,0.,0.), )
v9 = allVertices.findAt((.5,.5,0.), )

# Create a copy of the feature sketch to modify.

tmpSketch = model.ConstrainedSketch('tmp', feature.sketch)
v, d = tmpSketch.vertices, tmpSketch.dimensions

# Create some dictionaries to hold results. Seed the
# dictionaries with the theoretical results.

dispData, maxMomentData, minMomentData = {}, {}, {}
dispData['Theoretical'] = ((90, -.001478), (80, -.001409),
    (60, -0.000932), (40, -0.000349), (30, -0.000148))
maxMomentData['Theoretical'] = ((90, 0.0479), (80, 0.0486),
    (60, 0.0425), (40, 0.0281), (30, 0.0191))
minMomentData['Theoretical'] = ((90, 0.0479), (80, 0.0448),
    (60, 0.0333), (40, 0.0180), (30, 0.0108))
    
# Loop over the parameters to perform the parameter study.

for elemCode in elemTypeCodes:

    # Convert the element type codes to strings.

    elemName = repr(elemCode)
    dispData[elemName], maxMomentData[elemName], \
        minMomentData[elemName] = [], [], []

    # Set the element type.
    
    elemType = mesh.ElemType(elemCode=elemCode,
        elemLibrary=STANDARD)
    assembly.setElementType(regions=(instance.faces,), 
        elemTypes=(elemType,))
    
    for angle in angles:  
    
        # Skew the geometry and regenerate the mesh.
        assembly.deleteMesh(regions=(instance,))

        d[0].setValues(value=angle, )
        feature.setValues(sketch=tmpSketch)
        part.regenerate()
        assembly.regenerate()
        assembly.setLogicalCorners(
            region=face1, corners=(v1,v2,v9,v8))
        assembly.setLogicalCorners(
            region=face2, corners=(v2,v3,v4,v9))
        assembly.setLogicalCorners(
            region=face3, corners=(v9,v4,v5,v6))
        assembly.setLogicalCorners(
            region=face4, corners=(v8,v9,v6,v7))
        assembly.generateMesh(regions=(instance,))
        
        # Run the job, then process the results.
        
        job.submit()
        job.waitForCompletion()
        print ('Completed job for %s at %s degrees' % (elemName, angle))
        disp, maxMoment, minMoment = getResults()
        dispData[elemName].append((angle, disp))
        maxMomentData[elemName].append((angle, maxMoment))
        minMomentData[elemName].append((angle, minMoment))
        
# Plot the results.

createXYPlot((10,10), 'Skew 1', 'Displacement - 4x4 Mesh',
    dispData)
createXYPlot((160,10), 'Skew 2', 'Max Moment - 4x4 Mesh',
    maxMomentData)
createXYPlot((310,10), 'Skew 3', 'Min Moment - 4x4 Mesh',
    minMomentData)



from abaqus.CustomKernel.CommandRegister import CommandRegister  # noqa # pylint: disable=unused-import
from abaqus.CustomKernel.RegisteredDictionary import RegisteredDictionary  # noqa # pylint: disable=unused-import
from abaqus.CustomKernel.RegisteredList import RegisteredList  # noqa # pylint: disable=unused-import
from abaqus.CustomKernel.RegisteredTuple import RegisteredTuple  # noqa # pylint: disable=unused-import
from abaqus.CustomKernel.RepositorySupport import RepositorySupport  # noqa # pylint: disable=unused-import



"""The Highlight commands are used to highlight objects in the current viewport and to
remove the highlighting. 

"""
from abqpy.decorators import abaqus_function_doc


@abaqus_function_doc
def highlight(object):
    """This method highlights an object in the current viewport.

    .. note::
        This function can be accessed by::

            highlight

    Parameters
    ----------
    object
        An object specifying the object in the current viewport to be highlighted. You can
        specify only a single object. Abaqus/CAE highlights only the edges of a face when
        highlighting a surface and a face together. The following objects are supported:

        - **For the MDB**

            - ConstrainedSketchVertex
            - Edge
            - Face
            - Surface
            - Cell
            - Node
            - Element
            - Element face
            - Element edge
            - Feature
            - Datum
            - Instance
            - Set
            - Load
            - Boundary condition
            - Predefined field
            - Display group
        - **For the ODB**

            - Node
            - Element
            - Display group

    Returns
    -------
    """
    # TODO: Implement this method.
    ...


@abaqus_function_doc
def unhighlight(object):
    """This method removes highlighting from an object in the current viewport.

    .. note::
        This function can be accessed by::

            unhighlight

    Parameters
    ----------
    object
        An object specifying the object in the current viewport from which the highlighting will
        be removed. You can specify only a single object. See highlight for a list of supported
        objects.

    Returns
    -------
    """
    # TODO: Implement this method.
    ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .Amplitude import Amplitude
from ..UtilityAndView.abaqusConstants import (
    ABSOLUTE_VALUE,
    ACCELERATION,
    Boolean,
    EVENT_ACCELERATION,
    OFF,
    STEP,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class SpectrumAmplitude(Amplitude):
    """The SpectrumAmplitude object defines the spectrum of responses for displacement,
    velocity, or acceleration to be used in a response spectrum analysis.
    The SpectrumAmplitude object is derived from the Amplitude object.

    .. note::
        This object can be accessed by::

            import amplitude
            mdb.models[name].amplitudes[name]
            import odbAmplitude
            session.odbs[name].amplitudes[name]

        The corresponding analysis keywords are:

        - SPECTRUM
    """

    #: A String specifying the repository key.
    name: str

    #: A SymbolicConstant specifying the method for specifying the spectrum. Possible values
    #: are DEFINE and CALCULATE.
    method: SymbolicConstant

    #: A sequence of sequences of Floats specifying the magnitude, frequency, and damping
    #: values.
    data: tuple

    #: A SymbolicConstant specifying the units used for specifying the spectrum. Possible
    #: values are DISPLACEMENT, VELOCITY, ACCELERATION, and GRAVITY. The default value is
    #: ACCELERATION.
    specificationUnits: SymbolicConstant = ACCELERATION

    #: A SymbolicConstant specifying the units used to describe the dynamic event in the
    #: amplitude used for the calculation. Possible values are EVENT_DISPLACEMENT,
    #: EVENT_VELOCITY, EVENT_ACCELERATION, and EVENT_GRAVITY. The default value is
    #: EVENT_ACCELERATION.
    eventUnits: SymbolicConstant = EVENT_ACCELERATION

    #: A SymbolicConstant specifying the solution method for the dynamic equations. Possible
    #: values are ABSOLUTE_VALUE and RELATIVE_VALUE. The default value is ABSOLUTE_VALUE.
    solution: SymbolicConstant = ABSOLUTE_VALUE

    #: A Float specifying the implicit time increment used to calculate the spectrum. This
    #: argument is required when the **method** = CALCULATE. The default value is 0.0.
    timeIncrement: float = 0

    #: A Float specifying the acceleration due to gravity. This argument applies only when
    #: **specificationUnits** = GRAVITY or*eventUnits* = GRAVITY. The default value is 1.0.
    gravity: float = 1

    #: A Boolean specifying whether to calculate the spectrum for only the specified range of
    #: critical damping values or a list of values. If **criticalDamping** = ON, the spectrum is
    #: calculated only for the specified range of critical damping values. If **criticalDamping**
    #: = OFF, the spectrum is calculated for a list of damping values. The default value is
    #: OFF.
    criticalDamping: Boolean = OFF

    #: A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
    #: and TOTAL. The default value is STEP.
    timeSpan: SymbolicConstant = STEP

    #: A String specifying the name of the amplitude that describes the dynamic event used to
    #: calculate the spectrum. The default value is an empty string.
    amplitude: str = ""

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        method: Literal[C.DEFINE, C.CALCULATE],
        data: tuple,
        specificationUnits: Literal[C.ACCELERATION, C.VELOCITY, C.GRAVITY, C.DISPLACEMENT] = ACCELERATION,
        eventUnits: Literal[
            C.EVENT_DISPLACEMENT, C.EVENT_ACCELERATION, C.EVENT_GRAVITY, C.EVENT_VELOCITY
        ] = EVENT_ACCELERATION,
        solution: Literal[C.ABSOLUTE_VALUE, C.RELATIVE_VALUE] = ABSOLUTE_VALUE,
        timeIncrement: float = 0,
        gravity: float = 1,
        criticalDamping: Boolean = OFF,
        timeSpan: Literal[C.STEP, C.TOTAL] = STEP,
        amplitude: str = "",
    ):
        """This method creates a SpectrumAmplitude object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SpectrumAmplitude
                session.odbs[name].SpectrumAmplitude

        Parameters
        ----------
        name
            A String specifying the repository key.
        method
            A SymbolicConstant specifying the method for specifying the spectrum. Possible values
            are DEFINE and CALCULATE.
        data
            A sequence of sequences of Floats specifying the magnitude, frequency, and damping
            values.
        specificationUnits
            A SymbolicConstant specifying the units used for specifying the spectrum. Possible
            values are DISPLACEMENT, VELOCITY, ACCELERATION, and GRAVITY. The default value is
            ACCELERATION.
        eventUnits
            A SymbolicConstant specifying the units used to describe the dynamic event in the
            amplitude used for the calculation. Possible values are EVENT_DISPLACEMENT,
            EVENT_VELOCITY, EVENT_ACCELERATION, and EVENT_GRAVITY. The default value is
            EVENT_ACCELERATION.
        solution
            A SymbolicConstant specifying the solution method for the dynamic equations. Possible
            values are ABSOLUTE_VALUE and RELATIVE_VALUE. The default value is ABSOLUTE_VALUE.
        timeIncrement
            A Float specifying the implicit time increment used to calculate the spectrum. This
            argument is required when the **method** = CALCULATE. The default value is 0.0.
        gravity
            A Float specifying the acceleration due to gravity. This argument applies only when
            **specificationUnits** = GRAVITY or*eventUnits* = GRAVITY. The default value is 1.0.
        criticalDamping
            A Boolean specifying whether to calculate the spectrum for only the specified range of
            critical damping values or a list of values. If **criticalDamping** = ON, the spectrum is
            calculated only for the specified range of critical damping values. If **criticalDamping**
            = OFF, the spectrum is calculated for a list of damping values. The default value is
            OFF.
        timeSpan
            A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
            and TOTAL. The default value is STEP.
        amplitude
            A String specifying the name of the amplitude that describes the dynamic event used to
            calculate the spectrum. The default value is an empty string.

        Returns
        -------
        SpectrumAmplitude
            A SpectrumAmplitude object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        specificationUnits: Literal[C.ACCELERATION, C.VELOCITY, C.GRAVITY, C.DISPLACEMENT] = ACCELERATION,
        eventUnits: Literal[
            C.EVENT_DISPLACEMENT, C.EVENT_ACCELERATION, C.EVENT_GRAVITY, C.EVENT_VELOCITY
        ] = EVENT_ACCELERATION,
        solution: Literal[C.ABSOLUTE_VALUE, C.RELATIVE_VALUE] = ABSOLUTE_VALUE,
        timeIncrement: float = 0,
        gravity: float = 1,
        criticalDamping: Boolean = OFF,
        timeSpan: Literal[C.STEP, C.TOTAL] = STEP,
        amplitude: str = "",
    ):
        """This method modifies the SpectrumAmplitude object.

        Parameters
        ----------
        specificationUnits
            A SymbolicConstant specifying the units used for specifying the spectrum. Possible
            values are DISPLACEMENT, VELOCITY, ACCELERATION, and GRAVITY. The default value is
            ACCELERATION.
        eventUnits
            A SymbolicConstant specifying the units used to describe the dynamic event in the
            amplitude used for the calculation. Possible values are EVENT_DISPLACEMENT,
            EVENT_VELOCITY, EVENT_ACCELERATION, and EVENT_GRAVITY. The default value is
            EVENT_ACCELERATION.
        solution
            A SymbolicConstant specifying the solution method for the dynamic equations. Possible
            values are ABSOLUTE_VALUE and RELATIVE_VALUE. The default value is ABSOLUTE_VALUE.
        timeIncrement
            A Float specifying the implicit time increment used to calculate the spectrum. This
            argument is required when the **method** = CALCULATE. The default value is 0.0.
        gravity
            A Float specifying the acceleration due to gravity. This argument applies only when
            **specificationUnits** = GRAVITY or*eventUnits* = GRAVITY. The default value is 1.0.
        criticalDamping
            A Boolean specifying whether to calculate the spectrum for only the specified range of
            critical damping values or a list of values. If **criticalDamping** = ON, the spectrum is
            calculated only for the specified range of critical damping values. If **criticalDamping**
            = OFF, the spectrum is calculated for a list of damping values. The default value is
            OFF.
        timeSpan
            A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
            and TOTAL. The default value is STEP.
        amplitude
            A String specifying the name of the amplitude that describes the dynamic event used to
            calculate the spectrum. The default value is an empty string.

        Raises
        ------
        RangeError
        """
        ...



"""The following commands are used to request data entry from a user.

"""
from typing import overload

from abqpy.decorators import abaqus_function_doc


@abaqus_function_doc
@overload
def getInput(prompt: str, default: str = ""):
    r"""This method is used to obtain a single input from a user from a dialog box. The method
    displays a modal dialog box with a text field prefaced by the specified label. The user
    enters data in the text field or accepts the optional default value that is displayed in
    the text field. When the user clicks the OK button, the getInput method reads the data
    from the dialog box and closes the dialog box. You can use a \t separator in the label
    argument to provide a tooltip from the dialog box; for example::

        getInput('Enter a number:\tEnter the number of nodes to delete')

    .. note::
        You cannot use a script that contains getInput if you are running the script from
        the command line and passing the script name to the command line options **-start**,
        **-script**, **-replay**, or **-noGUI**.

    .. note::
        This function can be accessed by::

            getInput

    Parameters
    ----------
    prompt
        A String specifying the text field in the dialog box.
    default
        A String specifying a default value to be displayed in the text field in the dialog box.

    Returns
    -------
    str
        A String or None if the user clicks Cancel.
    """
    ...


@abaqus_function_doc
@overload
def getInput(fields: tuple, label: str = "", dialogTitle: str = ""):
    """This method is used to obtain multiple inputs from a user from a dialog box. The method
    displays a modal dialog box with a column of text fields prefaced by the specified
    labels. The user enters data in the text fields or accepts the optional default values
    that are displayed in the text field. When the user clicks the OK button, the getInputs
    method reads the data from the dialog box and closes the dialog box.
    Note:You cannot use a script that contains getInputs if you are running the script from
    the command line and passing the script name to the command line options **-start**,
    **-script**, **-replay**, or **-noGUI**.

    .. note::
        This function can be accessed by::

            getInputs

    Parameters
    ----------
    fields
        A sequence of sequence of Strings specifying the text fields in the dialog box. Each
        inner sequence is a pair of Strings specifying a label that describes a text field along
        with a default value that appears in the text field. If the field has no default value,
        you should enter an empty string in the second string in the pair.
    label
        A String specifying a label to be placed at the top of the dialog box. The default is an
        empty string, indicating that no label will be shown.
    dialogTitle
        A String specifying the text to be shown in the title bar of the dialog box. The default
        is Get Inputs.

    Returns
    -------
    Sequence[str]
        A sequence of Strings representing the data in each of the text fields in the dialog
        box. If the user clicks the Cancel button, the method returns a sequence of None
        objects.
    """
    ...


@abaqus_function_doc
def getInput(*args, **kwargs):
    ...


@abaqus_function_doc
def getWarningReply(message: str, buttons: str):
    r"""This method is used to obtain a reply from a user from a warning dialog box. The method
    displays a modal warning dialog box with a message and standard buttons. The user clicks
    the one of the standard buttons, the getWarningReply returns the corresponding button
    value and closes the dialog box. You can use a separator in the message argument to
    provide a multi-line message in the warning dialog box; for example::

        from abaqus import *
        getWarningReply('Out of disk space!\nOkay to continue', (YES, NO))

    .. note::
        You cannot use a script that contains getWarningReply if you are running the script
        from the command line and passing the script name to the command line options
        **-start**, **-script**, **-replay**, or **-noGUI**.

    .. note::
        This function can be accessed by::

            getWarningReply

    Parameters
    ----------
    message
        A String specifying the message in the warning dialog box.
    buttons
        A Sequence of standard buttons to be displayed in the warning dialog box. Permissible
        values are YES, NO, YES_TO_ALL and CANCEL.

    Returns
    -------
    The button that the user clicks.
    """
    ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.abaqusConstants import Boolean, OFF, ON, SHADED, SOLID, SymbolicConstant, UNSYMMETRIC, VERY_THIN
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class PlyStackPlotOptions(_OptionsBase):
    """The PlyStackPlotOptions object stores values and attributes associated with aViewport
    object. The PlyStackPlotOptions object has no constructor command. Abaqus creates the
    *detailPlotOptions.plyStackPlotPlotOptions* member whenever a Viewport is created.

    .. note::
        This object can be accessed by::

            session.viewports[name].detailPlotOptions.plyStackPlotOptions
    """

    #: A SymbolicConstant specifying how the plies in the viewport are rendered. Possible
    #: values are WIRE_FRAME, FILLED, and SHADED. The default value is SHADED.
    renderStyle: SymbolicConstant = SHADED

    #: A Boolean specifying whether to draw the edges for the plies. The default value is OFF.
    showEdges: Boolean = OFF

    #: A SymbolicConstant specifying the edge line style. Possible values are SOLID, DASHED,
    #: DOTTED, and DOT_DASH. The default value is SOLID.
    edgeStyle: SymbolicConstant = SOLID

    #: A SymbolicConstant specifying the edge line thickness. Possible values are VERY_THIN,
    #: THIN, MEDIUM, and THICK. The default value is VERY_THIN.
    edgeThickness: SymbolicConstant = VERY_THIN

    #: A SymbolicConstant specifying whether to display the plies in an unsymmetric staircase
    #: fashion or symmetric staircase fashion. Possible values are UNSYMMETRIC and SYMMETRIC.
    #: The default value is UNSYMMETRIC.
    plyDisplay: SymbolicConstant = UNSYMMETRIC

    #: An Int specifying the start ply. The default value is 1.
    startLayer: int = 1

    #: An Int specifying the number of plies to show. The default value is 30.
    numLayers: int = 30

    #: A String specifying the ply color for the even numbered plies. The numbering is based on
    #: the displayed plies and not on the actual ply number in the layup. The default value is
    #: "#A2B3FF".
    evenNumPlyColor: str = ""

    #: A String specifying the ply color for the odd numbered plies. The numbering is based on
    #: the displayed plies and not on the actual ply number in the layup. The default value is
    #: "#ECC9AD".
    oddNumPlyColor: str = ""

    #: A Boolean specifying whether to show the fibers. The default value is ON.
    showFibers: Boolean = ON

    #: A String specifying the fiber color. The default value is "Red".
    fiberColor: str = ""

    #: A SymbolicConstant specifying the fiber style. Possible values are SOLID, DASHED,
    #: DOTTED, and DOT_DASH. The default value is SOLID.
    fiberStyle: SymbolicConstant = SOLID

    #: A SymbolicConstant specifying the fiber thickness. Possible values are VERY_THIN, THIN,
    #: MEDIUM, and THICK. The default value is VERY_THIN.
    fiberThickness: SymbolicConstant = VERY_THIN

    #: A Float specifying the spacing between the fibers. The default value is 0.1.
    fiberSpacing: float = 0.1

    #: A Boolean specifying whether to show the reference surface. The default value is OFF.
    showReferencePlane: Boolean = OFF

    #: A String specifying the reference surface color. The default value is "White".
    referenceSurfaceColor: str = ""

    #: A Boolean specifying whether to set translucency. The default value is OFF.
    translucency: Boolean = OFF

    #: A Float specifying the translucency factor when **translucency** = ON. Possible values are
    #: 0.0≤ **translucencyFactor** ≤ 1.0. The default value is 0.3.
    translucencyFactor: float = 0.3

    #: A Boolean specifying whether to use depth sorting. The default value is OFF.
    translucencySort: Boolean = OFF

    #: A Boolean specifying whether to show the reference outline. The default value is OFF.
    showReferenceOutline: Boolean = OFF

    #: A String specifying the reference outline color. The default value is "Red".
    referenceOutlineColor: str = ""

    #: A SymbolicConstant specifying the reference outline style. Possible values are SOLID,
    #: DASHED, DOTTED, and DOT_DASH. The default value is SOLID.
    referenceOutlineStyle: SymbolicConstant = SOLID

    #: A SymbolicConstant specifying the reference outline thickness. Possible values are
    #: VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.
    referenceOutlineThickness: SymbolicConstant = VERY_THIN

    #: A String specifying the font for all the labels. The default value is
    #: “-*-verdana-medium-r-normal-*-*-80-*-*-p-*-*-*” on Windows and
    #: “-*-verdana-medium-r-normal-*-*-90-*-*-p-*-*-*” on Linux platforms. The default value is
    #: an empty string.
    allLabelsFont: str = ""

    #: A Boolean specifying whether to show the material names. The default value is OFF.
    showMaterialNames: Boolean = OFF

    #: A String specifying the material names text color. The default value is "Green".
    materialNamesColor: str = ""

    #: A Boolean specifying whether to show the orientation names or the fiber angles. The
    #: default value is OFF.
    showOrientationNames: Boolean = OFF

    #: A String specifying the orientation names fiber angles text color. The default value is
    #: "Orange".
    orientationNamesColor: str = ""

    #: A Boolean specifying whether to show the state block. The default value is ON.
    showStateBlock: Boolean = ON

    #: A String specifying the state block text color. The default value is "White".
    stateBlockColor: str = ""

    #: A Boolean specifying whether to show the ply names. The default value is ON.
    showPlyNames: Boolean = ON

    #: A String specifying the ply names text color. The default value is "Yellow".
    plyNamesColor: str = ""

    #: A Boolean specifying whether to show the thickness labels. The default value is ON.
    showThicknessLabels: Boolean = ON

    #: A String specifying the thickness labels text color. The default value is "Red".
    thicknessLabelsColor: str = ""

    #: A Boolean specifying whether to show the through thickness integration points. The
    #: default value is OFF.
    showIntPoints: Boolean = OFF

    #: A String specifying the through thickness integration points color. The default value is
    #: "Blue".
    intPointsColor: str = ""

    #: A Float specifying the size of the ply in the X-direction. The default value is 1.5.
    sizeX: float = 1.5

    #: A Float specifying the size of the ply in the Y-direction. The default value is 1.0.
    sizeY: float = 1.0

    #: A Float specifying the size of the ply in the Z-direction. The default value is 0.8.
    sizeZ: float = 0.8

    @abaqus_method_doc
    def setValues(
        self,
        *,
        renderStyle: Literal[C.WIRE_FRAME, C.FILLED, C.SHADED] = SHADED,
        showEdges: Boolean = OFF,
        edgeStyle: Literal[C.DASHED, C.SOLID, C.DOT_DASH, C.DOTTED] = SOLID,
        edgeThickness: Literal[C.THIN, C.THICK, C.VERY_THIN, C.MEDIUM] = VERY_THIN,
        plyDisplay: Literal[C.SYMMETRIC, C.UNSYMMETRIC] = UNSYMMETRIC,
        startLayer: int = 1,
        numLayers: int = 30,
        evenNumPlyColor: str = "",
        oddNumPlyColor: str = "",
        showFibers: Boolean = ON,
        fiberColor: str = "",
        fiberStyle: Literal[C.DASHED, C.SOLID, C.DOT_DASH, C.DOTTED] = SOLID,
        fiberThickness: Literal[C.THIN, C.THICK, C.VERY_THIN, C.MEDIUM] = VERY_THIN,
        fiberSpacing: float = 0.1,
        showReferencePlane: Boolean = OFF,
        referenceSurfaceColor: str = "",
        translucency: Boolean = OFF,
        translucencyFactor: float = 0.3,
        translucencySort: Boolean = OFF,
        showReferenceOutline: Boolean = OFF,
        referenceOutlineColor: str = "",
        referenceOutlineStyle: Literal[C.DASHED, C.SOLID, C.DOT_DASH, C.DOTTED] = SOLID,
        referenceOutlineThickness: Literal[C.THIN, C.THICK, C.VERY_THIN, C.MEDIUM] = VERY_THIN,
        allLabelsFont: str = "",
        showMaterialNames: Boolean = OFF,
        materialNamesColor: str = "",
        showOrientationNames: Boolean = OFF,
        orientationNamesColor: str = "",
        showStateBlock: Boolean = ON,
        stateBlockColor: str = "",
        showPlyNames: Boolean = ON,
        plyNamesColor: str = "",
        showThicknessLabels: Boolean = ON,
        thicknessLabelsColor: str = "",
        showIntPoints: Boolean = OFF,
        intPointsColor: str = "",
        sizeX: float = 1.5,
        sizeY: float = 1.0,
        sizeZ: float = 0.8,
    ):
        """This method modifies the PlyStackPlotOptions object.

        Parameters
        ----------
        renderStyle
            A SymbolicConstant specifying how the plies in the viewport are rendered. Possible
            values are WIRE_FRAME, FILLED, and SHADED. The default value is SHADED.
        showEdges
            A Boolean specifying whether to draw the edges for the plies. The default value is OFF.
        edgeStyle
            A SymbolicConstant specifying the edge line style. Possible values are SOLID, DASHED,
            DOTTED, and DOT_DASH. The default value is SOLID.
        edgeThickness
            A SymbolicConstant specifying the edge line thickness. Possible values are VERY_THIN,
            THIN, MEDIUM, and THICK. The default value is VERY_THIN.
        plyDisplay
            A SymbolicConstant specifying whether to display the plies in an unsymmetric staircase
            fashion or symmetric staircase fashion. Possible values are UNSYMMETRIC and SYMMETRIC.
            The default value is UNSYMMETRIC.
        startLayer
            An Int specifying the start ply. The default value is 1.
        numLayers
            An Int specifying the number of plies to show. The default value is 30.
        evenNumPlyColor
            A String specifying the ply color for the even numbered plies. The numbering is based on
            the displayed plies and not on the actual ply number in the layup. The default value is
            "#A2B3FF".
        oddNumPlyColor
            A String specifying the ply color for the odd numbered plies. The numbering is based on
            the displayed plies and not on the actual ply number in the layup. The default value is
            "#ECC9AD".
        showFibers
            A Boolean specifying whether to show the fibers. The default value is ON.
        fiberColor
            A String specifying the fiber color. The default value is "Red".
        fiberStyle
            A SymbolicConstant specifying the fiber style. Possible values are SOLID, DASHED,
            DOTTED, and DOT_DASH. The default value is SOLID.
        fiberThickness
            A SymbolicConstant specifying the fiber thickness. Possible values are VERY_THIN, THIN,
            MEDIUM, and THICK. The default value is VERY_THIN.
        fiberSpacing
            A Float specifying the spacing between the fibers. The default value is 0.1.
        showReferencePlane
            A Boolean specifying whether to show the reference surface. The default value is OFF.
        referenceSurfaceColor
            A String specifying the reference surface color. The default value is "White".
        translucency
            A Boolean specifying whether to set translucency. The default value is OFF.
        translucencyFactor
            A Float specifying the translucency factor when **translucency** = ON. Possible values are
            0.0≤ **translucencyFactor** ≤ 1.0. The default value is 0.3.
        translucencySort
            A Boolean specifying whether to use depth sorting. The default value is OFF.
        showReferenceOutline
            A Boolean specifying whether to show the reference outline. The default value is OFF.
        referenceOutlineColor
            A String specifying the reference outline color. The default value is "Red".
        referenceOutlineStyle
            A SymbolicConstant specifying the reference outline style. Possible values are SOLID,
            DASHED, DOTTED, and DOT_DASH. The default value is SOLID.
        referenceOutlineThickness
            A SymbolicConstant specifying the reference outline thickness. Possible values are
            VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.
        allLabelsFont
            A String specifying the font for all the labels. The default value is
            “-*-verdana-medium-r-normal-*-*-80-*-*-p-*-*-*” on Windows and
            “-*-verdana-medium-r-normal-*-*-90-*-*-p-*-*-*” on Linux platforms. The default value is
            an empty string.
        showMaterialNames
            A Boolean specifying whether to show the material names. The default value is OFF.
        materialNamesColor
            A String specifying the material names text color. The default value is "Green".
        showOrientationNames
            A Boolean specifying whether to show the orientation names or the fiber angles. The
            default value is OFF.
        orientationNamesColor
            A String specifying the orientation names fiber angles text color. The default value is
            "Orange".
        showStateBlock
            A Boolean specifying whether to show the state block. The default value is ON.
        stateBlockColor
            A String specifying the state block text color. The default value is "White".
        showPlyNames
            A Boolean specifying whether to show the ply names. The default value is ON.
        plyNamesColor
            A String specifying the ply names text color. The default value is "Yellow".
        showThicknessLabels
            A Boolean specifying whether to show the thickness labels. The default value is ON.
        thicknessLabelsColor
            A String specifying the thickness labels text color. The default value is "Red".
        showIntPoints
            A Boolean specifying whether to show the through thickness integration points. The
            default value is OFF.
        intPointsColor
            A String specifying the through thickness integration points color. The default value is
            "Blue".
        sizeX
            A Float specifying the size of the ply in the X-direction. The default value is 1.5.
        sizeY
            A Float specifying the size of the ply in the Y-direction. The default value is 1.0.
        sizeZ
            A Float specifying the size of the ply in the Z-direction. The default value is 0.8.
        """
        super().setValues(
            allLabelsFont=allLabelsFont,
            edgeStyle=edgeStyle,
            edgeThickness=edgeThickness,
            evenNumPlyColor=evenNumPlyColor,
            fiberColor=fiberColor,
            fiberSpacing=fiberSpacing,
            fiberStyle=fiberStyle,
            fiberThickness=fiberThickness,
            intPointsColor=intPointsColor,
            materialNamesColor=materialNamesColor,
            numLayers=numLayers,
            oddNumPlyColor=oddNumPlyColor,
            orientationNamesColor=orientationNamesColor,
            plyDisplay=plyDisplay,
            plyNamesColor=plyNamesColor,
            referenceOutlineColor=referenceOutlineColor,
            referenceOutlineStyle=referenceOutlineStyle,
            referenceOutlineThickness=referenceOutlineThickness,
            referenceSurfaceColor=referenceSurfaceColor,
            renderStyle=renderStyle,
            showEdges=showEdges,
            showFibers=showFibers,
            showIntPoints=showIntPoints,
            showMaterialNames=showMaterialNames,
            showOrientationNames=showOrientationNames,
            showPlyNames=showPlyNames,
            showReferenceOutline=showReferenceOutline,
            showReferencePlane=showReferencePlane,
            showStateBlock=showStateBlock,
            showThicknessLabels=showThicknessLabels,
            sizeX=sizeX,
            sizeY=sizeY,
            sizeZ=sizeZ,
            startLayer=startLayer,
            stateBlockColor=stateBlockColor,
            thicknessLabelsColor=thicknessLabelsColor,
            translucency=translucency,
            translucencyFactor=translucencyFactor,
            translucencySort=translucencySort,
        )



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .SectionCategory import SectionCategory


@abaqus_class_doc
class OdbMeshElement:
    """OdbMeshElement objects are created with the part.addElements or rootAssembly.addElements
    methods.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].parts[name].elements[i]
            session.odbs[name].parts[name].elementSets[name].elements[i]
            session.odbs[name].parts[name].nodeSets[name].elements[i]
            session.odbs[name].parts[name].surfaces[name].elements[i]
            session.odbs[name].rootAssembly.elements[i]
            session.odbs[name].rootAssembly.elementSets[name].elements[i]
            session.odbs[name].rootAssembly.instances[name].elements[i]
            session.odbs[name].rootAssembly.instances[name].elementSets[name].elements[i]
            session.odbs[name].rootAssembly.instances[name].nodeSets[name].elements[i]
            session.odbs[name].rootAssembly.instances[name].surfaces[name].elements[i]
            session.odbs[name].rootAssembly.nodeSets[name].elements[i]
            session.odbs[name].rootAssembly.surfaces[name].elements[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.elements[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.elementSets[name].elements[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.nodeSets[name].elements[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.surfaces[name].elements[i]
    """

    #: An Int specifying the element label.
    label: Optional[int] = None

    #: A String specifying the element type.
    type: str = ""

    #: A SectionCategory object specifying the element section properties.
    sectionCategory: Optional[SectionCategory] = None

    #: A tuple of Ints specifying the element connectivity. For connector elements connected to
    #: ground, the other node is repeated in the connectivity data. The position of the ground
    #: node cannot be ascertained. This is a limitation. It is important to note the difference
    #: with MeshElement object of MDB where the connectivity is node indices instead of node
    #: labels.
    connectivity: Optional[int] = None

    #: A tuple of Strings specifying the instance names for nodes in the element connectivity.
    instanceNames: tuple = ()

    #: A String specifying the instance name.
    instanceName: str = ""



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import Boolean, ON


@abaqus_class_doc
class PredefinedFieldDisplayOptions:
    """The PredefinedFieldDisplayOptions object stores settings that specify how assemblies are
    to be displayed in a particular viewport when
    session.viewports[name].assemblyDisplay.predefinedFields=ON
    The PredefinedFieldDisplayOptions object has no constructor. When you create a new
    viewport, the settings are copied from the current viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].assemblyDisplay.predefinedFieldOptions
            session.viewports[name].layers[name].assemblyDisplay.predefinedFieldOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        temperatureField: Boolean = ON,
        velocityField: Boolean = ON,
        generalField: Boolean = ON,
        stressField: Boolean = ON,
        hardeningField: Boolean = ON,
    ):
        """This method modifies the PredefinedFieldDisplayOptions object.

        Parameters
        ----------
        temperatureField
            A Boolean specifying whether temperature field symbols are shown. The default value is
            ON.
        velocityField
            A Boolean specifying whether translational velocity symbols are shown. The default value
            is ON.
        generalField
            A Boolean specifying whether general field symbols are shown. The default value is ON.
        stressField
            A Boolean specifying whether stress field symbols are shown. The default value is ON.
        hardeningField
            A Boolean specifying whether hardening field symbols are shown. The default value is ON.

        Raises
        ------
        RangeError
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .BoundaryConditionState import BoundaryConditionState
from ..UtilityAndView.abaqusConstants import Boolean, FREE, ON, SymbolicConstant


@abaqus_class_doc
class EulerianMotionBCState(BoundaryConditionState):
    """The EulerianMotionBCState object stores the propagating data for an Eulerian mesh motion
    boundary condition in a step. One instance of this object is created internally by the
    EulerianMotionBC object for each step. The instance is also deleted internally by the
    EulerianMotionBC object.
    The EulerianMotionBCState object has no constructor or methods.
    The EulerianMotionBCState object is derived from the BoundaryConditionState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - EULERIAN MESH MOTION
    """

    #: A SymbolicConstant specifying the 1-direction translational constraint on the center of
    #: the Eulerian mesh. Possible values are FREE and FIXED. The default value is FREE.
    ctrPosition1: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the 2-direction translational constraint on the center of
    #: the Eulerian mesh. Possible values are FREE and FIXED. The default value is FREE.
    ctrPosition2: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the 3-direction translational constraint on the center of
    #: the Eulerian mesh. Possible values are FREE and FIXED. The default value is FREE.
    ctrPosition3: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the translational constraint on the positive (maximum)
    #: bounds of the mesh in the 1 direction. Possible values are FREE and FIXED. The default
    #: value is FREE.
    posPosition1: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the translational constraint on the positive (maximum)
    #: bounds of the mesh in the 2 direction. Possible values are FREE and FIXED. The default
    #: value is FREE.
    posPosition2: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the translational constraint on the positive (maximum)
    #: bounds of the mesh in the 3 direction. Possible values are FREE and FIXED. The default
    #: value is FREE.
    posPosition3: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the translational constraint on the negative (minimum)
    #: bounds of the mesh in the 1 direction. Possible values are FREE and FIXED. The default
    #: value is FREE.
    negPosition1: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the translational constraint on the negative (minimum)
    #: bounds of the mesh in the 2 direction. Possible values are FREE and FIXED. The default
    #: value is FREE.
    negPosition2: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the translational constraint on the negative (minimum)
    #: bounds of the mesh in the 3 direction. Possible values are FREE and FIXED. The default
    #: value is FREE.
    negPosition3: SymbolicConstant = FREE

    #: None or a Float specifying the upper bounds on the allowable scaling of the mesh in the
    #: 1 direction. If **expansionRatio1** = None, then there is no upper limit. The default value
    #: is None.
    expansionRatio1: Optional[float] = None

    #: None or a Float specifying the upper bounds on the allowable scaling of the mesh in the
    #: 2 direction. If **expansionRatio2** = None, then there is no upper limit. The default value
    #: is None.
    expansionRatio2: Optional[float] = None

    #: None or a Float specifying the upper bounds on the allowable scaling of the mesh in the
    #: 3 direction. If **expansionRatio3** = None, then there is no upper limit. The default value
    #: is None.
    expansionRatio3: Optional[float] = None

    #: A Float specifying the lower bounds on the allowable scaling of the mesh in the 1
    #: direction. The default value is 0.0.
    contractRatio1: float = 0

    #: A Float specifying the lower bounds on the allowable scaling of the mesh in the 2
    #: direction. The default value is 0.0.
    contractRatio2: float = 0

    #: A Float specifying the lower bounds on the allowable scaling of the mesh in the 3
    #: direction. The default value is 0.0.
    contractRatio3: float = 0

    #: A Boolean specifying whether the mesh is allowed to contract . The default value is ON.
    allowContraction: Boolean = ON

    #: A Float specifying the maximum change in allowed aspect ratio (for any of the three mesh
    #: aspects, 1-2, 2-3, 3-1). The default value is 10.0.
    aspectLimit: float = 10

    #: A Float specifying the multiplier for the mesh nodal velocity limit. The default value
    #: is 1.01.
    vmaxFactor: float = 1

    #: A Float specifying the lower bounds on the volume fraction when determining which nodes
    #: to include in the surface bounding box calculation for an Eulerian material surface.
    #: This argument applies only when **followRegion** = False. The default value is 0.5.
    volThreshold: float = 0

    #: None or a Float specifying the buffer between the surface box and the Eulerian section
    #: mesh bounding box. The default value is 2.0.
    bufferSize: float = 2

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .Load import Load
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, OFF, SymbolicConstant, UNIFORM, UNSET
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class RotationalBodyForce(Load):
    """The RotationalBodyForce object stores the data for a rotational body force.
    The RotationalBodyForce object is derived from the Load object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the load is distributed spatially. Possible values are
    #: UNIFORM and FIELD. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A Boolean specifying whether or not the effect of the load is centrifugal. The default
    #: value is OFF. Note: At least one of **centrifugal** or **rotaryAcceleration** must be
    #: specified and only one must have the value ON.
    centrifugal: Boolean = OFF

    #: A Boolean specifying whether or not the effect of the load is rotary acceleration. The
    #: default value is OFF. Note: At least one of **centrifugal** or **rotaryAcceleration** must be
    #: specified and only one must have the value ON.
    rotaryAcceleration: Boolean = OFF

    #: A tuple of Floats specifying the first point on the axis of rotation for the load.
    point1: Optional[float] = None

    #: A tuple of Floats specifying the second point on the axis of rotation for the load.
    point2: Optional[float] = None

    #: A String specifying the name of the AnalyticalField object associated with this load.
    #: The **field** argument applies only when **distributionType** = FIELD. The default value is an
    #: empty string.
    field: str = ""

    #: A Region object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        magnitude: float,
        point1: tuple,
        point2: tuple,
        distributionType: Literal[C.FIELD, C.UNIFORM] = UNIFORM,
        field: str = "",
        centrifugal: Boolean = OFF,
        rotaryAcceleration: Boolean = OFF,
        amplitude: str = UNSET,
    ):
        """This method creates a RotationalBodyForce object.

        .. note::
            This function can be accessed by::

                mdb.models[name].RotationalBodyForce

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the load is created. This must be the
            first analysis step name.
        region
            A Region object specifying the region to which the load is applied.
        magnitude
            A Float specifying the load magnitude.
        point1
            A sequence of Floats specifying the first point on the axis of rotation for the load.
        point2
            A sequence of Floats specifying the second point on the axis of rotation for the load.
        distributionType
            A SymbolicConstant specifying how the load is distributed spatially. Possible values are
            UNIFORM and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        centrifugal
            A Boolean specifying whether or not the effect of the load is centrifugal. The default
            value is OFF. Note: At least one of **centrifugal** or **rotaryAcceleration** must be
            specified and only one must have the value ON.
        rotaryAcceleration
            A Boolean specifying whether or not the effect of the load is rotary acceleration. The
            default value is OFF. Note: At least one of **centrifugal** or **rotaryAcceleration** must be
            specified and only one must have the value ON.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.

        Returns
        -------
        RotationalBodyForce
            A RotationalBodyForce object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        distributionType: Literal[C.FIELD, C.UNIFORM] = UNIFORM,
        field: str = "",
        centrifugal: Boolean = OFF,
        rotaryAcceleration: Boolean = OFF,
        amplitude: str = UNSET,
    ):
        """This method modifies the data for an existing RotationalBodyForce object in the step
        where it is created.

        Parameters
        ----------
        distributionType
            A SymbolicConstant specifying how the load is distributed spatially. Possible values are
            UNIFORM and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        centrifugal
            A Boolean specifying whether or not the effect of the load is centrifugal. The default
            value is OFF. Note: At least one of **centrifugal** or **rotaryAcceleration** must be
            specified and only one must have the value ON.
        rotaryAcceleration
            A Boolean specifying whether or not the effect of the load is rotary acceleration. The
            default value is OFF. Note: At least one of **centrifugal** or **rotaryAcceleration** must be
            specified and only one must have the value ON.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, magnitude: Optional[float] = None, amplitude: str = ""):
        """This method modifies the propagating data for an existing RotationalBodyForce object in
        the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the load is modified.
        magnitude
            A Float specifying the load magnitude.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous static analysis step. FREED should be used if
            the load is changed to have no amplitude reference. You should provide the **amplitude**
            argument only if it is valid for the specified step.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class CoriolisForceState(LoadState):
    """The CoriolisForceState object stores the propagating data of a coriolis force in a step.
    One instance of this object is created internally by the CoriolisForce object for each
    step. The instance is also deleted internally by the CoriolisForce object.
    The CoriolisForceState object has no constructor or methods.
    The CoriolisForceState object is derived from the LoadState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - DLOAD
    """

    #: A Float specifying the load magnitude.
    magnitude: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the load magnitude. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    magnitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from math import *  # noqa # pylint: disable=unused-import

import abqpy.abaqus

abqpy.abaqus.run(cae=False)

from abaqus.Odb.OdbCommands import *  # noqa # pylint: disable=unused-import
from abaqus.UtilityAndView.BackwardCompatibility import BackwardCompatibility

backwardCompatibility = BackwardCompatibility()



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import NONE


@abaqus_class_doc
class QuantityType:
    """The QuantityType object is used to store attributes defining the physical dimension and
    label of the quantity type to be associated with an XYData or an AxisData object.
    QuantityType objects are automatically created when creating XYData objects from the
    odb.
    QuantityType objects can be created using the methods described below.

    .. note::
        This object can be accessed by::

            import visualization
            session.charts[name].axes1[i].axisData.curves[i].data.axis1QuantityType
            session.charts[name].axes1[i].axisData.curves[i].data.axis2QuantityType
            session.charts[name].axes1[i].axisData.quantityType
            session.charts[name].axes2[i].axisData.curves[i].data.axis1QuantityType
            session.charts[name].axes2[i].axisData.curves[i].data.axis2QuantityType
            session.charts[name].axes2[i].axisData.quantityType
            session.charts[name].curves[name].data.axis1QuantityType
            session.charts[name].curves[name].data.axis2QuantityType
            session.curves[name].data.axis1QuantityType
            session.curves[name].data.axis2QuantityType
            session.defaultChartOptions.defaultAxis1Options.axisData.curves[i].data.axis1QuantityType
            session.defaultChartOptions.defaultAxis1Options.axisData.curves[i].data.axis2QuantityType
            session.defaultChartOptions.defaultAxis1Options.axisData.quantityType
            session.defaultChartOptions.defaultAxis2Options.axisData.curves[i].data.axis1QuantityType
            session.defaultChartOptions.defaultAxis2Options.axisData.curves[i].data.axis2QuantityType
            session.defaultChartOptions.defaultAxis2Options.axisData.quantityType
            import odbAccess
            session.odbs[name].userData.axis1QuantityType
            session.odbs[name].userData.axis2QuantityType
            import visualization
            import xyPlot
            session.odbs[name].userData.xyDataObjects[name].axis1QuantityType
            session.odbs[name].userData.xyDataObjects[name].axis2QuantityType
            session.xyDataObjects[name].axis1QuantityType
            session.xyDataObjects[name].axis2QuantityType
            session.xyPlots[name].charts[name].axes1[i].axisData.curves[i].data.axis1QuantityType
            session.xyPlots[name].charts[name].axes1[i].axisData.curves[i].data.axis2QuantityType
            session.xyPlots[name].charts[name].axes1[i].axisData.quantityType
            session.xyPlots[name].charts[name].axes2[i].axisData.curves[i].data.axis1QuantityType
            session.xyPlots[name].charts[name].axes2[i].axisData.curves[i].data.axis2QuantityType
            session.xyPlots[name].charts[name].axes2[i].axisData.quantityType
            session.xyPlots[name].charts[name].curves[name].data.axis1QuantityType
            session.xyPlots[name].charts[name].curves[name].data.axis2QuantityType
            session.xyPlots[name].curves[name].data.axis1QuantityType
            session.xyPlots[name].curves[name].data.axis2QuantityType
    """

    #: A String specifying the label for this quantity type.
    label: str = ""

    #: A SymbolicConstant specifying the physical dimension of the axis. Possible values are:
    #:
    #: - NONE.
    #: - ACCELERATION.
    #: - ACOUSTIC_INTENSITY.
    #: - ANGLE.
    #: - ANGULAR_MOMENTUM.
    #: - ARC_LENGTH.
    #: - AREA.
    #: - AREA_VELOCITY_SQUARED, specifying "Velocity squared per area".
    #: - BIMOMENT.
    #: - CURVATURE.
    #: - CORIOLIS_LOAD.
    #: - DAMAGE.
    #: - DAMAGE_CRITERION.
    #: - DENSITY.
    #: - DENSITY_ROTATIONAL_ACCELERATION, specifying "Density * Angular acceleration".
    #: - DISPLACEMENT.
    #: - ECURRENT_AREA_TIME, specifying "Time integrated electric current per area".
    #: - ELECTRIC_CHARGE.
    #: - ELECTRIC_CURRENT.
    #: - ELECTRIC_CURRENT_AREA, specifying "Electric current per unit area".
    #: - ELECTRIC_POTENTIAL.
    #: - ENERGY.
    #: - ENERGY_DENSITY.
    #: - ENERGY_RELEASE_RATE.
    #: - EPOTENTIAL_GRADIENT, specifying "Electric potential gradient".
    #: - FREQUENCY.
    #: - FORCE.
    #: - FORCE_VOLUME, specifying "Force per volume".
    #: - HEAT_FLUX.
    #: - HEAT_FLUX_AREA, specifying "Heat flux per area".
    #: - HEAT_FLUX_RATE.
    #: - HEAT_FLUX_VOLUME, specifying "Heat flux per volume".
    #: - LENGTH.
    #: - LINEAR_PRESSURE.
    #: - LUMIN, specifying "Luminous intensity".
    #: - MASS.
    #: - MASS_FLOW_AREA, specifying "Mass flow per area".
    #: - MASS_FLOW_AREA_RATE, specifying "Mass flow rate per area".
    #: - MASS_FLOW_RATE.
    #: - MODE_NUMBER.
    #: - MOMENT.
    #: - NUMBER.
    #: - PATH.
    #: - PHASE.
    #: - POSITION.
    #: - PRESSURE.
    #: - PRESSURE_GRADIENT.
    #: - RATE.
    #: - ROTARY_INERTIA.
    #: - ROTATIONAL_ACCELERATION.
    #: - ROTATIONAL_VELOCITY.
    #: - STATUS.
    #: - STRAIN.
    #: - STRAIN_RATE.
    #: - STRESS.
    #: - STRESS_INTENS_FACTOR, specifying "Stress intensity factor".
    #: - SUBSTANCE, specifying "Amount of substance".
    #: - TEMPERATURE.
    #: - THICKNESS.
    #: - TIME.
    #: - TIME_INCREMENT.
    #: - TIME_HEAT_FLUX, specifying "Time integrated heat flux".
    #: - TIME_HEAT_FLUX_AREA, specifying "Time integrated heat flux per area".
    #: - TIME_VOLUME, specifying "Time integrated volume".
    #: - TIME_VOLUME_FLUX, specifying "Time integrated volume flux per area".
    #: - TWIST.
    #: - VELOCITY.
    #: - VELOCITY_SQUARED.
    #: - VOLUME.
    #: - VOLUME_FLUX.
    #: - VOLUME_FLUX_AREA, specifying "Volume flux per area".
    #: - VOLUME_FRACTION.
    #:
    #: The default value is NONE
    type: Literal[
        C.NONE,
        C.ACCELERATION,
        C.ACOUSTIC_INTENSITY,
        C.ANGLE,
        C.ANGULAR_MOMENTUM,
        C.ARC_LENGTH,
        C.AREA,
        C.AREA_VELOCITY_SQUARED,
        C.BIMOMENT,
        C.CURVATURE,
        C.CORIOLIS_LOAD,
        C.DAMAGE,
        C.DAMAGE_CRITERION,
        C.DENSITY,
        C.DENSITY_ROTATIONAL_ACCELERATION,
        C.DISPLACEMENT,
        C.ECURRENT_AREA_TIME,
        C.ELECTRIC_CHARGE,
        C.ELECTRIC_CURRENT,
        C.ELECTRIC_CURRENT_AREA,
        C.ELECTRIC_POTENTIAL,
        C.ENERGY,
        C.ENERGY_DENSITY,
        C.ENERGY_RELEASE_RATE,
        C.EPOTENTIAL_GRADIENT,
        C.FREQUENCY,
        C.FORCE,
        C.FORCE_VOLUME,
        C.HEAT_FLUX,
        C.HEAT_FLUX_AREA,
        C.HEAT_FLUX_RATE,
        C.HEAT_FLUX_VOLUME,
        C.LENGTH,
        C.LINEAR_PRESSURE,
        C.LUMIN,
        C.MASS,
        C.MASS_FLOW_AREA,
        C.MASS_FLOW_AREA_RATE,
        C.MASS_FLOW_RATE,
        C.MODE_NUMBER,
        C.MOMENT,
        C.NUMBER,
        C.PATH,
        C.PHASE,
        C.POSITION,
        C.PRESSURE,
        C.PRESSURE_GRADIENT,
        C.RATE,
        C.ROTARY_INERTIA,
        C.ROTATIONAL_ACCELERATION,
        C.ROTATIONAL_VELOCITY,
        C.STATUS,
        C.STRAIN,
        C.STRAIN_RATE,
        C.STRESS,
        C.STRESS_INTENS_FACTOR,
        C.SUBSTANCE,
        C.TEMPERATURE,
        C.THICKNESS,
        C.TIME,
        C.TIME_INCREMENT,
        C.TIME_HEAT_FLUX,
        C.TIME_HEAT_FLUX_AREA,
        C.TIME_VOLUME,
        C.TIME_VOLUME_FLUX,
        C.TWIST,
        C.VELOCITY,
        C.VELOCITY_SQUARED,
        C.VOLUME,
        C.VOLUME_FLUX,
        C.VOLUME_FLUX_AREA,
        C.VOLUME_FRACTION,
    ] = NONE

    @abaqus_method_doc
    def __init__(
        self,
        type: Literal[
            C.NONE,
            C.ACCELERATION,
            C.ACOUSTIC_INTENSITY,
            C.ANGLE,
            C.ANGULAR_MOMENTUM,
            C.ARC_LENGTH,
            C.AREA,
            C.AREA_VELOCITY_SQUARED,
            C.BIMOMENT,
            C.CURVATURE,
            C.CORIOLIS_LOAD,
            C.DAMAGE,
            C.DAMAGE_CRITERION,
            C.DENSITY,
            C.DENSITY_ROTATIONAL_ACCELERATION,
            C.DISPLACEMENT,
            C.ECURRENT_AREA_TIME,
            C.ELECTRIC_CHARGE,
            C.ELECTRIC_CURRENT,
            C.ELECTRIC_CURRENT_AREA,
            C.ELECTRIC_POTENTIAL,
            C.ENERGY,
            C.ENERGY_DENSITY,
            C.ENERGY_RELEASE_RATE,
            C.EPOTENTIAL_GRADIENT,
            C.FREQUENCY,
            C.FORCE,
            C.FORCE_VOLUME,
            C.HEAT_FLUX,
            C.HEAT_FLUX_AREA,
            C.HEAT_FLUX_RATE,
            C.HEAT_FLUX_VOLUME,
            C.LENGTH,
            C.LINEAR_PRESSURE,
            C.LUMIN,
            C.MASS,
            C.MASS_FLOW_AREA,
            C.MASS_FLOW_AREA_RATE,
            C.MASS_FLOW_RATE,
            C.MODE_NUMBER,
            C.MOMENT,
            C.NUMBER,
            C.PATH,
            C.PHASE,
            C.POSITION,
            C.PRESSURE,
            C.PRESSURE_GRADIENT,
            C.RATE,
            C.ROTARY_INERTIA,
            C.ROTATIONAL_ACCELERATION,
            C.ROTATIONAL_VELOCITY,
            C.STATUS,
            C.STRAIN,
            C.STRAIN_RATE,
            C.STRESS,
            C.STRESS_INTENS_FACTOR,
            C.SUBSTANCE,
            C.TEMPERATURE,
            C.THICKNESS,
            C.TIME,
            C.TIME_INCREMENT,
            C.TIME_HEAT_FLUX,
            C.TIME_HEAT_FLUX_AREA,
            C.TIME_VOLUME,
            C.TIME_VOLUME_FLUX,
            C.TWIST,
            C.VELOCITY,
            C.VELOCITY_SQUARED,
            C.VOLUME,
            C.VOLUME_FLUX,
            C.VOLUME_FLUX_AREA,
            C.VOLUME_FRACTION,
        ],
        label: str = "",
    ):
        """This method creates a QuantityType object.

        .. note::
            This function can be accessed by::

                session.QuantityType
                xyPlot.QuantityType

        Parameters
        ----------
        type
            A SymbolicConstant specifying the physical dimension of the axis. Possible values are:

            - NONE.
            - ACCELERATION.
            - ACOUSTIC_INTENSITY.
            - ANGLE.
            - ANGULAR_MOMENTUM.
            - ARC_LENGTH.
            - AREA.
            - AREA_VELOCITY_SQUARED, specifying "Velocity squared per area".
            - BIMOMENT.
            - CURVATURE.
            - CORIOLIS_LOAD.
            - DAMAGE.
            - DAMAGE_CRITERION.
            - DENSITY.
            - DENSITY_ROTATIONAL_ACCELERATION, specifying "Density * Angular acceleration".
            - DISPLACEMENT.
            - ECURRENT_AREA_TIME, specifying "Time integrated electric current per area".
            - ELECTRIC_CHARGE.
            - ELECTRIC_CURRENT.
            - ELECTRIC_CURRENT_AREA, specifying "Electric current per unit area".
            - ELECTRIC_POTENTIAL.
            - ENERGY.
            - ENERGY_DENSITY.
            - ENERGY_RELEASE_RATE.
            - EPOTENTIAL_GRADIENT, specifying "Electric potential gradient".
            - FREQUENCY.
            - FORCE.
            - FORCE_VOLUME, specifying "Force per volume".
            - HEAT_FLUX.
            - HEAT_FLUX_AREA, specifying "Heat flux per area".
            - HEAT_FLUX_RATE.
            - HEAT_FLUX_VOLUME, specifying "Heat flux per volume".
            - LENGTH.
            - LINEAR_PRESSURE.
            - LUMIN, specifying "Luminous intensity".
            - MASS.
            - MASS_FLOW_AREA, specifying "Mass flow per area".
            - MASS_FLOW_AREA_RATE, specifying "Mass flow rate per area".
            - MASS_FLOW_RATE.
            - MODE_NUMBER.
            - MOMENT.
            - NUMBER.
            - PATH.
            - PHASE.
            - POSITION.
            - PRESSURE.
            - PRESSURE_GRADIENT.
            - RATE.
            - ROTARY_INERTIA.
            - ROTATIONAL_ACCELERATION.
            - ROTATIONAL_VELOCITY.
            - STATUS.
            - STRAIN.
            - STRAIN_RATE.
            - STRESS.
            - STRESS_INTENS_FACTOR, specifying "Stress intensity factor".
            - SUBSTANCE, specifying "Amount of substance".
            - TEMPERATURE.
            - THICKNESS.
            - TIME.
            - TIME_INCREMENT.
            - TIME_HEAT_FLUX, specifying "Time integrated heat flux".
            - TIME_HEAT_FLUX_AREA, specifying "Time integrated heat flux per area".
            - TIME_VOLUME, specifying "Time integrated volume".
            - TIME_VOLUME_FLUX, specifying "Time integrated volume flux per area".
            - TWIST.
            - VELOCITY.
            - VELOCITY_SQUARED.
            - VOLUME.
            - VOLUME_FLUX.
            - VOLUME_FLUX_AREA, specifying "Volume flux per area".
            - VOLUME_FRACTION.
        label
            A String specifying the label for this quantity type.

        Returns
        -------
        QuantityType
            A QuantityType object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the QuantityType object."""
        ...



from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .ConstrainedSketchGeometry import ConstrainedSketchGeometry


@abaqus_class_doc
class ArcByCenterEnds(ConstrainedSketchGeometry):
    @abaqus_method_doc
    def __init__(
        center: Sequence[float],
        point1: Sequence[float],
        point2: Sequence[float],
    ):
        """This method constructs an arc using a center point and two vertices. The Arc object is
        added to the geometry repository of the ConstrainedSketch object. The arc is created in
        a clockwise fashion from **point1** to **point2**.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ArcByCenterEnds

        Parameters
        ----------
        center
            A pair of Floats specifying the center point of the arc.
        point1
            A pair of Floats specifying the first endpoint of the arc.
        point2
            A pair of Floats specifying the second endpoint of the arc.

        Returns
        -------
        ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object (None if the arc cannot be created).

        Raises
        ------
        If incompatible data are given, the second endpoint is ignored.

        """
        ...



from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..ConstrainedSketchBase import ConstrainedSketchBase
from ...UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class ConstrainedSketchGeometryModel(ConstrainedSketchBase):
    """A ConstrainedSketch object contains the entities that are used to create a sketch. The
    objects include ConstrainedSketchGeometry objects contained in the ConstrainedSketchGeometry Repository,
    such as Line, Arc, and Spline. ConstrainedSketchVertex, ConstrainedSketchDimension, ConstrainedSketchConstraint, and ConstrainedSketchParameter objects are
    contained in their respective repositories.

    .. note::
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name]
    """

    @abaqus_method_doc
    def Arc3Points(self, point1: Sequence[float], point2: Sequence[float], point3: Sequence[float]):
        """This method constructs an arc using a two endpoints and an intermediate third point on
        the arc.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].Arc3Points

        Parameters
        ----------
        point1
            A pair of Floats specifying the first endpoint of the arc.
        point2
            A pair of Floats specifying the second endpoint of the arc.
        point3
            A pair of Floats specifying the third point on the arc.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def ArcByCenterEnds(
        self,
        center: Sequence[float],
        point1: Sequence[float],
        point2: Sequence[float],
    ):
        """This method constructs an arc using a center point and two vertices. The Arc object is
        added to the geometry repository of the ConstrainedSketch object. The arc is created in
        a clockwise fashion from **point1** to **point2**.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ArcByCenterEnds

        Parameters
        ----------
        center
            A pair of Floats specifying the center point of the arc.
        point1
            A pair of Floats specifying the first endpoint of the arc.
        point2
            A pair of Floats specifying the second endpoint of the arc.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object

        Raises
        ------
        If incompatible data are given, the second endpoint is ignored
        """
        ...

    @abaqus_method_doc
    def ArcByStartEndTangent(self, point1: Sequence[float], point2: Sequence[float], vector: tuple):
        """This method constructs an arc using two vertices. The Arc object is added to the
        geometry repository of the ConstrainedSketch object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ArcByStartEndTangent

        Parameters
        ----------
        point1
            A pair of Floats specifying the first endpoint of the arc.
        point2
            A pair of Floats specifying the second endpoint of the arc.
        vector
            A sequence of two Floats specifying the start direction for constructing the arc.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def CircleByCenterPerimeter(self, center: Sequence[float], point1: Sequence[float]):
        """This method constructs a circle using a center point and a point on the perimeter. The
        circle is added to the geometry repository of the ConstrainedSketch object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].CircleByCenterPerimeter

        Parameters
        ----------
        center
            A pair of Floats specifying the center point of the circle.
        point1
            A pair of Floats specifying a point on the perimeter of the circle.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def ConstructionCircleByCenterPerimeter(self, center: Sequence[float], point1: Sequence[float]):
        """This method constructs a construction circle using a center point and a point on the
        perimeter. The circle is added to the geometry repository of the ConstrainedSketch
        object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ConstructionCircleByCenterPerimeter

        Parameters
        ----------
        center
            A pair of Floats specifying the center point of the construction circle.
        point1
            A pair of Floats specifying a point on the perimeter of the construction circle.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def EllipseByCenterPerimeter(
        self, center: Sequence[float], axisPoint1: Sequence[float], axisPoint2: Sequence[float]
    ):
        """This method constructs an ellipse using a center point, a major axis point, and a minor
        axis point. The ellipse is added to the geometry repository of the ConstrainedSketch
        object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].EllipseByCenterPerimeter

        Parameters
        ----------
        center
            A pair of Floats specifying the center point of the ellipse.
        axisPoint1
            A pair of Floats specifying the major or minor axis point of the ellipse.
        axisPoint2
            A pair of Floats specifying the minor or major axis point of the ellipse.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def FilletByRadius(
        self,
        radius: float,
        curve1: "ConstrainedSketchGeometryModel",
        nearPoint1: Sequence[float],
        curve2: "ConstrainedSketchGeometryModel",
        nearPoint2: Sequence[float],
    ):
        """This method constructs a fillet arc of a given radius between two curves. The fillet is
        added to the geometry repository of the ConstrainedSketch object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].FilletByRadius

        Parameters
        ----------
        radius
            A Float specifying the radius of the fillet arc. Possible values are Floats > 0.
        curve1
            A ConstrainedSketchGeometry object specifying the first curve.
        nearPoint1
            A pair of Floats specifying a point on the sketch near where the user wishes the fillet
            to intersect with **curve1**. This point does not need to be on*curve1*; it is used as a
            hint to draw the fillet.
        curve2
            A ConstrainedSketchGeometry object specifying the second curve.
        nearPoint2
            A pair of Floats specifying a point on the sketch near where the user wishes the fillet
            to intersect with **curve2**. This point does not need to be on **curve2**; it is used as a
            hint to draw the fillet.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object

        Raises
        ------
        Range Error: cannot construct the Fillet specified
            If the radius given cannot be used to create a fillet between the two curves given.
        """
        ...

    @abaqus_method_doc
    def Line(self, point1: Sequence[float], point2: Sequence[float]):
        """This method creates a line between two given points.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].Line

        Parameters
        ----------
        point1
            A pair of Floats specifying the first endpoint.
        point2
            A pair of Floats specifying the second endpoint.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def ConstructionLine(self, point1: Sequence[float], point2: Sequence[float]):
        """This method creates an oblique construction line that runs between two given points.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ConstructionLine

        Parameters
        ----------
        point1
            A pair of Floats specifying the first endpoint.
        point2
            A pair of Floats specifying the second endpoint.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def Spline(self, points: tuple, constrainPoints: Boolean = True):
        """This method creates a spline curve running through a sequence of points.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].Spline

        Parameters
        ----------
        points
            A sequence of pairs of Floats specifying the points through which the spline passes.
        constrainPoints
            A Boolean that determines whether the points given are to constrained to always remain
            on the Spline. The default is True. For a large sequence of **points**, significant
            performance gains may be achieved by setting the value to False.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def Spot(self, point: Sequence[float]):
        """This method creates a spot construction point located at the specified coordinates. The
        spot is added to the vertex repository of the ConstrainedSketch object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].Spot

        Parameters
        ----------
        point
            A pair of Floats specifying the coordinates of the spot construction point.

        Returns
        -------
        geometry: ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object
        """
        ...

    @abaqus_method_doc
    def getVertices(self):
        """This method returns an list of ConstrainedSketchVertex objects which are a part of the
        given ConstrainedSketchGeometry object.

        Returns
        -------
        vertices: List[ConstrainedSketchVertex]
            A list of ConstrainedSketchVertex objects
        """
        ...

    @abaqus_method_doc
    def getSize(self):
        """This method returns the length of the given ConstrainedSketchGeometry object.

        Returns
        -------
        length: int
            The length of the given ConstrainedSketchGeometry
        """
        ...

    @abaqus_method_doc
    def getPointAtDistance(self, point: Sequence[float], distance: str, percentage: Boolean = OFF):
        """This method returns a point offset along the given ConstrainedSketchGeometry from the
        given end by a specified arc length distance or a percentage of the total length of the
        ConstrainedSketchGeometry object.

        Parameters
        ----------
        point
            A pair of Floats specifying the point from which the distance is to be measured.
        distance
            A float specifying the arc length distance along the ConstrainedSketchGeometry from the
            **point** at which the required point is situated.
        percentage
            A Boolean that specifies if the **distance** is an absolute distance or is a fraction
            relative to the length of the ConstrainedSketchGeometry object.

        Returns
        -------
        points: Sequence[float]
            A pair of floats representing the point along the edge
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import Boolean, OFF, ON
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class ImageAnimationOptions(_OptionsBase):
    """The ImageAnimationOptions object is used to store values and attributes associated with
    saving viewport animations. The ImageAnimationOptions object has no constructor. Abaqus
    creates the **imageAnimationOptions** member when the animation module is imported.

    .. note::
        This object can be accessed by::

            import animation
            session.imageAnimationOptions
    """

    #: An Int specifying the frame rate to record on the saved animation file. The effective
    #: frame rate in frames per second will be obtained by dividing the given frame rate by the
    #: time scale.
    frameRate: Optional[int] = None

    #: An Int specifying the time scale to apply to the frame rate.
    vpDecorations: Boolean = ON

    #: A Boolean specifying whether to capture viewport backgrounds. The default value is OFF.
    vpBackground: Boolean = OFF

    #: A Boolean specifying whether to capture the view compass. The default value is OFF.
    compass: Boolean = OFF

    @abaqus_method_doc
    def setValues(
        self,
        frameRate: Optional[int] = None,
        timeScale: Optional[int] = None,
        vpDecorations: Boolean = ON,
        vpBackground: Boolean = OFF,
        compass: Boolean = OFF,
    ):
        """This method modifies the ImageAnimationOptions object.

        Parameters
        ----------
        frameRate
            An Int specifying the frame rate to record on the saved animation file. The effective
            frame rate in frames per second will be obtained by dividing the given frame rate by the
            time scale.
        timeScale
            An Int specifying the time scale to apply to the frame rate.
        vpDecorations
            A Boolean specifying whether to capture the viewport border and title. The default value
            is ON.
        vpBackground
            A Boolean specifying whether to capture viewport backgrounds. The default value is OFF.
        compass
            A Boolean specifying whether to capture the view compass. The default value is OFF.
        """
        super().setValues(
            frameRate=frameRate,
            timeScale=timeScale,
            vpDecorations=vpDecorations,
            vpBackground=vpBackground,
            compass=compass,
        )
